@@ -1,4 +1,5 @@
 {-# OPTIONS_GHC -XTypeSynonymInstances -XScopedTypeVariables #-}
+{-# OPTIONS_GHC -fno-warn-incomplete-patterns #-}
 -----------------------------------------------------------------------------
 --
 -- Module       :  IDE.Menu
@@ -53,7 +54,6 @@ import Graphics.UI.Gtk
         menuItemGetSubmenu, menuShellAppend, onActivateLeaf,
         menuItemNewWithLabel, menuNew, Packing(..), ToolbarStyle(..),
         PositionType(..), on, IconSize(..))
-import Control.Monad.Reader
 import System.FilePath
 import Data.Version
 import Prelude hiding (catch)
@@ -67,6 +67,7 @@ import IDE.Pane.Preferences
 import IDE.Pane.PackageFlags
 import IDE.Pane.PackageEditor
 import IDE.Pane.Errors
+import IDE.Pane.Search
 import IDE.Package
 import IDE.Pane.Log
 import IDE.Session
@@ -97,11 +98,15 @@ import IDE.Pane.Breakpoints
 import IDE.Workspaces
 import IDE.Statusbar
 import IDE.Pane.Workspace
-import IDE.Pane.Variables (fillVariablesList)
+import IDE.Pane.Variables (fillVariablesListQuiet)
 import IDE.Pane.Trace (fillTraceList)
 import IDE.PaneGroups
-import IDE.Pane.Search (getSearch, setChoices, searchMetaGUI)
+import IDE.Pane.Search (getSearch, IDESearch(..))
 import IDE.Pane.Grep (getGrep)
+import IDE.Pane.Files (getFiles)
+import Control.Monad.IO.Class (MonadIO(..))
+import Control.Monad (unless, when)
+import Control.Monad.Trans.Reader (ask)
 
 --
 -- | The Actions known to the system (they can be activated by keystrokes or menus)
@@ -224,8 +229,8 @@ mkActions =
     ,AD "ResolveErrors" "Resol_ve Errors" (Just "Resolve 'Hidden package' and 'Not in scope' errors by adding the necessary dependancies or imports") Nothing
         resolveErrors [] False
 
-    ,AD "InstallPackage" "_Install" Nothing Nothing
-        (packageTry_ packageInstall) [] False
+    ,AD "InstallDependenciesPackage" "_Install Dependencies" (Just "Install the package's dependencies from the hackage server") Nothing
+        (packageTry_ packageInstallDependencies) [] False
     ,AD "RegisterPackage" "_Register" Nothing Nothing
         (packageTry_ packageRegister) [] False
     ,AD "TestPackage" "Test" Nothing Nothing
@@ -331,6 +336,8 @@ mkActions =
         showDebugger [] False
     ,AD "ShowSearch" "Search" Nothing Nothing
         (getSearch Nothing  >>= \ p -> displayPane p False) [] False
+    ,AD "ShowFiles" "Files" Nothing Nothing
+        (getFiles Nothing  >>= \ p -> displayPane p False) [] False
     ,AD "ShowGrep" "Grep" Nothing Nothing
         (getGrep Nothing  >>= \ p -> displayPane p False) [] False
     ,AD "ShowErrors" "Errors" Nothing Nothing
@@ -409,6 +416,8 @@ mkActions =
 
     ,AD "BackgroundBuildToggled" "_BackgroundBuild" (Just "Build in the background and report errors") (Just "ide_build")
         backgroundBuildToggled [] True
+    ,AD "RunUnitTestsToggled" "_RunUnitTests" (Just "Run unit tests when building") (Just "gtk-apply")
+        runUnitTestsToggled [] True
     ,AD "MakeModeToggled" "_MakeMode" (Just "Make dependent packages") (Just "ide_make")
         makeModeToggled [] True
     ,AD "DebugToggled" "_Debug" (Just "Use GHCi debugger to build and run") (Just "ide_debug")
@@ -510,44 +519,46 @@ getMenuAndToolbars uiManager = do
 
 textPopupMenu :: IDERef -> Menu -> IO ()
 textPopupMenu ideR menu = do
+    let reflectIDE_ x = reflectIDE x ideR
     items <- containerGetChildren menu
     mi1 <- menuItemNewWithLabel "Eval"
-    mi1 `onActivateLeaf` (reflectIDE debugExecuteSelection ideR)
+    mi1 `onActivateLeaf` reflectIDE_ debugExecuteSelection
     menuShellAppend menu mi1
     mi11 <- menuItemNewWithLabel "Eval & Insert"
-    mi11 `onActivateLeaf` (reflectIDE debugExecuteAndShowSelection ideR)
+    mi11 `onActivateLeaf` reflectIDE_ debugExecuteAndShowSelection
     menuShellAppend menu mi11
     mi12 <- menuItemNewWithLabel "Step"
-    mi12 `onActivateLeaf` (reflectIDE debugStepExpression ideR)
+    mi12 `onActivateLeaf` reflectIDE_ debugStepExpression
     menuShellAppend menu mi12
     mi13 <- menuItemNewWithLabel "Trace"
-    mi13 `onActivateLeaf` (reflectIDE debugTraceExpression ideR)
+    mi13 `onActivateLeaf` reflectIDE_ debugTraceExpression
     menuShellAppend menu mi13
     mi16 <- menuItemNewWithLabel "Set Breakpoint"
-    mi16 `onActivateLeaf` (reflectIDE debugSetBreakpoint ideR)
+    mi16 `onActivateLeaf` reflectIDE_ debugSetBreakpoint
     menuShellAppend menu mi16
     sep1 <- separatorMenuItemNew
     menuShellAppend menu sep1
     mi14 <- menuItemNewWithLabel "Type"
-    mi14 `onActivateLeaf` (reflectIDE debugType ideR)
+    mi14 `onActivateLeaf` reflectIDE_ debugType
     menuShellAppend menu mi14
     mi141 <- menuItemNewWithLabel "Info"
-    mi141 `onActivateLeaf` (reflectIDE debugInformation ideR)
+    mi141 `onActivateLeaf` reflectIDE_ debugInformation
     menuShellAppend menu mi141
     mi15 <- menuItemNewWithLabel "Kind"
-    mi15 `onActivateLeaf` (reflectIDE debugKind ideR)
+    mi15 `onActivateLeaf` reflectIDE_ debugKind
     menuShellAppend menu mi15
     sep2 <- separatorMenuItemNew
     menuShellAppend menu sep2
     mi2 <- menuItemNewWithLabel "Find (text)"
-    mi2 `onActivateLeaf` (reflectIDE (editFindInc Initial) ideR)
+    mi2 `onActivateLeaf` reflectIDE_ (editFindInc Initial)
     menuShellAppend menu mi2
     mi3 <- menuItemNewWithLabel "Search (metadata)"
-    mi3 `onActivateLeaf` (reflectIDE (do
-        mbtext <- selectedText
-        case mbtext of
-            Just t  -> searchMetaGUI t
-            Nothing -> ideMessage  Normal "Select a text first") ideR)
+    mi3 `onActivateLeaf` (reflectIDE_ $
+            getSearch Nothing >>= (\search -> do
+                mbtext <- selectedText
+                case mbtext of
+                    Just t  ->  searchMetaGUI search t
+                    Nothing -> ideMessage  Normal "Select a text first"))
     menuShellAppend menu mi3
     let interpretingEntries = [castToWidget mi16]
     let interpretingSelEntries = [castToWidget mi1, castToWidget mi11, castToWidget mi12,
@@ -631,12 +642,15 @@ getActionsFor SensitivityForwardHist = getActionsFor' ["ViewHistoryForth"]
 getActionsFor SensitivityBackwardHist = getActionsFor' ["ViewHistoryBack"]
 getActionsFor SensitivityProjectActive = getActionsFor'
     ["EditPackage", "PackageFlags", "ConfigPackage", "BuildPackage"
-    ,"DocPackage", "CleanPackage", "CopyPackage", "RunPackage","InstallPackage"
+    ,"DocPackage", "CleanPackage", "CopyPackage", "RunPackage","InstallDependenciesPackage"
     ,"RegisterPackage", "TestPackage","SdistPackage"
     ,"OpenDocPackage","FileCloseAll"]
 getActionsFor SensitivityError = getActionsFor' ["NextError", "PreviousError"]
-getActionsFor SensitivityEditor = getActionsFor' ["EditUndo", "EditRedo", "EditGotoLine"
-    ,"EditComment", "EditUncomment", "EditShiftLeft", "EditShiftRight"]
+getActionsFor SensitivityEditor = getActionsFor' ["EditUndo", "EditRedo",
+        "EditGotoLine","EditComment", "EditUncomment",
+        "EditShiftLeft", "EditShiftRight","FileClose","ResolveErrors",
+        "OpenDocu"
+        ]
 getActionsFor SensitivityInterpreting = getActionsFor' ["QuitDebugger"]
 getActionsFor SensitivityWorkspaceOpen = return [] --TODO add here
 
@@ -694,6 +708,7 @@ instrumentWindow win prefs topWidget = do
         reflectIDE (do
             setCandyState (fst (sourceCandy prefs))
             setBackgroundBuildToggled (backgroundBuild prefs)
+            setRunUnitTests (runUnitTests prefs)
             setMakeModeToggled (makeMode prefs)) ideR
 
 instrumentSecWindow :: Window -> IDEAction
@@ -733,7 +748,6 @@ handleSpecialKeystrokes (Key { eventKeyName = name,  eventModifier = mods,
                 when bs (editKeystrokeCandy mbChar)
                 sk  <- readIDE specialKey
                 sks <- readIDE specialKeys
-                return True
                 case sk of
                     Nothing ->
                         case Map.lookup (keyVal,sort mods) sks of
@@ -763,23 +777,25 @@ handleSpecialKeystrokes (Key { eventKeyName = name,  eventModifier = mods,
     printMods (m:r) = show m ++ printMods r
 handleSpecialKeystrokes _ = return True
 
-setSymbol :: String -> IDEAction
-setSymbol symbol = do
+setSymbol :: String -> Bool -> IDEAction
+setSymbol symbol openSource = do
     currentInfo' <- getWorkspaceInfo
+    search <- getSearch Nothing
     case currentInfo' of
         Nothing -> return ()
         Just ((GenScopeC (PackScope _ symbolTable1)),(GenScopeC (PackScope _ symbolTable2))) ->
             case filter (not . isReexported) (getIdentifierDescr symbol symbolTable1 symbolTable2) of
                 []     -> return ()
-                a:[]   -> selectIdentifier a
+                a:[]   -> selectIdentifier a openSource
                 a:b:[] -> if isJust (dscMbModu a) && dscMbModu a == dscMbModu b &&
                             isNear (dscMbLocation a) (dscMbLocation b)
-                                then selectIdentifier a
-                                else setChoices [a,b]
-                l      -> setChoices l
+                                then selectIdentifier a openSource
+                                else setChoices search [a,b]
+                l      -> setChoices search l
   where
     isNear (Just a) (Just b) = abs (locationSLine a - locationSLine b) <= 3
     isNear _ _               = False
+
 --
 -- | Register handlers for IDE events
 --
@@ -790,9 +806,9 @@ registerLeksahEvents =    do
         (\e@(LogMessage s t)      -> getLog >>= \(log :: IDELog) -> liftIO $ appendLog log s t
                                                 >> return e)
     registerEvent stRef "SelectInfo"
-        (\ e@(SelectInfo str)     -> setSymbol str >> return e)
+        (\ e@(SelectInfo str gotoSource)     -> setSymbol str gotoSource >> return e)
     registerEvent stRef "SelectIdent"
-        (\ e@(SelectIdent id)     -> selectIdentifier id >> return e)
+        (\ e@(SelectIdent id)     -> selectIdentifier id False >> return e)
     registerEvent stRef "InfoChanged"
         (\ e@(InfoChanged b)      -> reloadKeepSelection b >> return e)
     registerEvent stRef "UpdateWorkspaceInfo"
@@ -805,7 +821,9 @@ registerLeksahEvents =    do
     registerEvent stRef "Sensitivity"
         (\ s@(Sensitivity h)      -> setSensitivity h >> return s)
     registerEvent stRef "SearchMeta"
-        (\ e@(SearchMeta string)  -> searchMetaGUI string >> return e)
+        (\ e@(SearchMeta string)  -> getSearch Nothing >>= (flip searchMetaGUI) string >> return e)
+    registerEvent stRef "StartFindInitial"
+        (\ e@(StartFindInitial)  -> editFindInc Initial >> return e)
     registerEvent stRef "LoadSession"
         (\ e@(LoadSession fp)     -> loadSession fp >> return e)
     registerEvent stRef "SaveSession"
@@ -813,7 +831,7 @@ registerLeksahEvents =    do
     registerEvent stRef "UpdateRecent"
         (\ e@UpdateRecent         -> updateRecentEntries >> return e)
     registerEvent stRef "VariablesChanged"
-        (\ e@VariablesChanged     -> fillVariablesList >> return e)
+        (\ e@VariablesChanged     -> fillVariablesListQuiet >> return e)
     registerEvent stRef "ErrorChanged"
         (\ e@ErrorChanged         -> postAsyncIDE fillErrorList >> return e)
     registerEvent stRef "CurrentErrorChanged"
@@ -828,6 +846,8 @@ registerLeksahEvents =    do
             selectBreak mbLogRef) >> return e)
     registerEvent stRef "TraceChanged"
         (\ e@TraceChanged         -> fillTraceList >> return e)
+    registerEvent stRef "GotoDefinition"
+        (\ e@(GotoDefinition descr)         -> goToDefinition descr >> return e)
     registerEvent stRef "GetTextPopup"
         (\ e@(GetTextPopup _)     -> return (GetTextPopup (Just textPopupMenu)))
     registerEvent stRef "StatusbarChanged"
@@ -835,3 +855,4 @@ registerLeksahEvents =    do
     return ()
 
 
+
