@@ -1,5 +1,6 @@
-{-# OPTIONS_GHC -XFlexibleContexts -XTypeSynonymInstances -XMultiParamTypeClasses
-    -XScopedTypeVariables -XCPP -XDeriveDataTypeable #-}
+{-# LANGUAGE FlexibleInstances, FlexibleContexts, TypeSynonymInstances,
+             MultiParamTypeClasses, ScopedTypeVariables, CPP,
+             DeriveDataTypeable #-}
 -----------------------------------------------------------------------------
 --
 -- Module      :  IDE.Core.State
@@ -45,6 +46,7 @@ module IDE.Core.State (
 
 ,   reifyIDE
 ,   reflectIDE
+,   reflectIDEI
 ,   catchIDE
 ,   postSyncIDE
 ,   postAsyncIDE
@@ -60,6 +62,7 @@ module IDE.Core.State (
 --,   deactivatePaneIfActive
 --,   closePane
 ,   activeProjectDir
+,   changePackage
 
 ,   liftYiControl
 ,   liftYi
@@ -76,10 +79,9 @@ import Graphics.UI.Gtk hiding (get)
 import Graphics.UI.Gtk.SourceView.SourceView ()
 
 import Data.IORef
-import Control.Monad.Reader hiding (liftIO)
 import Control.Exception
 import Prelude hiding (catch)
-import Control.Monad.State
+import Control.Monad.IO.Class (MonadIO, liftIO)
 import IDE.Core.Types
 import Graphics.UI.Frame.Panes
 import Graphics.UI.Frame.ViewFrame --hiding (notebookInsertOrdered)
@@ -90,9 +92,14 @@ import System.FilePath (dropFileName)
 import IDE.Core.CTypes
 import Control.Concurrent (forkIO)
 import IDE.Utils.Utils
-import qualified Data.Map as Map (lookup)
+import qualified Data.Map as Map (empty, lookup)
 import Data.Typeable(Typeable)
 import qualified IDE.YiConfig as Yi
+import Data.Enumerator (runIteratee, Iteratee(..))
+import qualified Data.Enumerator as E
+       (returnI, Step(..), yield, continue)
+import Control.Monad (liftM, when)
+import Control.Monad.Trans.Reader (ask, ReaderT(..))
 
 instance PaneMonad IDEM where
     getFrameState   =   readIDE frameState
@@ -284,6 +291,15 @@ reifyIDE = ReaderT
 reflectIDE :: IDEM a -> IDERef -> IO a
 reflectIDE c ideR = runReaderT c ideR
 
+reflectIDEI :: Iteratee a IDEM b -> IDERef -> Iteratee a IO b
+reflectIDEI c ideR = loop c where
+    loop x = do
+        s <- liftIO $ reflectIDE (runIteratee x) ideR
+        case s of
+            E.Continue f -> E.continue $ loop . f
+            E.Yield a b  -> E.yield a b
+            E.Error e    -> E.returnI $ E.Error e
+
 liftYiControl :: Yi.ControlM a -> IDEM a
 liftYiControl f = do
     control <- readIDE yiControl
@@ -384,5 +400,23 @@ deactivatePaneIfActive pane = do
                         then deactivatePane
                         else return ()
 
+changePackage :: IDEPackage -> IDEAction
+changePackage ideP@IDEPackage{ipdCabalFile = file} = do
+    oldWorkspace <- readIDE workspace
+    case oldWorkspace of
+        Nothing -> return ()
+        Just ws -> do
+            let ps = map exchange (wsPackages ws)
+            modifyIDE_ (\ide -> ide{workspace = Just ws {wsPackages = ps},
+                                    bufferProjCache = Map.empty})
+    mbActivePack <- readIDE activePack
+    case mbActivePack of
+        Just activePack | ipdCabalFile ideP == ipdCabalFile activePack ->
+            modifyIDE_ (\ide -> ide{activePack = Just ideP})
+        _ -> return ()
+    where
+        exchange p | ipdCabalFile p == file = ideP
+                   | otherwise              = p
+
 
 
