@@ -33,6 +33,7 @@ module IDE.Core.Types (
 ,   IDEAction
 ,   IDEEvent(..)
 ,   liftIDE
+,   (?>>=)
 
 ,   WorkspaceM
 ,   WorkspaceAction
@@ -88,11 +89,11 @@ import qualified IDE.YiConfig as Yi
 import Graphics.UI.Gtk
        (Window(..), KeyVal(..), Color(..), Menu(..), TreeView(..),
         ListStore(..), Toolbar(..))
-import Control.Monad.Reader
 import Data.Unique (newUnique, Unique(..))
 import Graphics.UI.Frame.Panes
 import Distribution.Package
     (PackageIdentifier(..), Dependency(..))
+import Distribution.PackageDescription (BuildInfo)
 import Data.Map (Map(..))
 import Data.Set (Set(..))
 import Distribution.ModuleName (ModuleName(..))
@@ -103,8 +104,7 @@ import Graphics.UI.Gtk.Gdk.Enums (Modifier(..))
 #endif
 import System.Time (ClockTime(..))
 import Distribution.Simple (Extension(..))
-import IDE.System.Process (ProcessHandle(..))
-import IDE.Utils.Tool (ToolState(..))
+import IDE.Utils.Tool (ToolState(..), ProcessHandle)
 import Data.IORef (writeIORef, readIORef, IORef(..))
 import Numeric (showHex)
 import Control.Event
@@ -115,6 +115,9 @@ import IDE.StrippedPrefs(RetrieveStrategy)
 import System.IO (Handle)
 import Distribution.Text(disp)
 import Text.PrettyPrint (render)
+import Control.Monad.Trans.Class (lift)
+import Control.Monad.IO.Class (liftIO)
+import Control.Monad.Trans.Reader (ReaderT(..))
 
 -- ---------------------------------------------------------------------
 -- IDE State
@@ -187,6 +190,13 @@ data IDEState =
 liftIDE :: IDEM a -> WorkspaceM a
 liftIDE = lift
 
+(?>>=) :: Monad m => (m (Maybe a)) -> (a -> m ()) -> m ()
+a ?>>= b = do
+    mA <- a
+    case mA of
+        Just v -> b v
+        Nothing -> return ()
+
 -- ---------------------------------------------------------------------
 -- Monad for functions that need an open workspace
 --
@@ -221,12 +231,14 @@ runDebug = runReaderT
 data IDEEvent  =
         InfoChanged Bool-- is it the initial = True else False
     |   UpdateWorkspaceInfo
-    |   SelectInfo String
+    |   SelectInfo String Bool -- navigate to source (== True)
     |   SelectIdent Descr
     |   LogMessage String LogTag
     |   RecordHistory GUIHistory
     |   Sensitivity [(SensitivityMask,Bool)]
     |   SearchMeta String
+    |   StartFindInitial
+    |   GotoDefinition Descr
     |   LoadSession FilePath
     |   SaveSession FilePath
     |   UpdateRecent
@@ -244,11 +256,13 @@ instance Event IDEEvent String where
     getSelector (InfoChanged _)         =   "InfoChanged"
     getSelector UpdateWorkspaceInfo     =   "UpdateWorkspaceInfo"
     getSelector (LogMessage _ _)        =   "LogMessage"
-    getSelector (SelectInfo _)          =   "SelectInfo"
+    getSelector (SelectInfo _ _)        =   "SelectInfo"
     getSelector (SelectIdent _)         =   "SelectIdent"
     getSelector (RecordHistory _)       =   "RecordHistory"
     getSelector (Sensitivity _)         =   "Sensitivity"
     getSelector (SearchMeta _)          =   "SearchMeta"
+    getSelector (StartFindInitial)      =   "StartFindInitial"
+    getSelector (GotoDefinition _)      =   "GotoDefinition"
     getSelector (LoadSession _)         =   "LoadSession"
     getSelector (SaveSession _)         =   "SaveSession"
     getSelector UpdateRecent            =   "UpdateRecent"
@@ -272,6 +286,9 @@ instance EventSource IDERef IDEEvent IDEM String where
     canTriggerEvent _ "Sensitivity"         = True
     canTriggerEvent _ "DescrChoice"         = True
     canTriggerEvent _ "SearchMeta"          = True
+    canTriggerEvent _ "StartFindInitial"    = True
+    canTriggerEvent _ "SearchSymbolDialog"  = True
+    canTriggerEvent _ "GotoDefinition"      = True
     canTriggerEvent _ "LoadSession"         = True
     canTriggerEvent _ "SaveSession"         = True
     canTriggerEvent _ "UpdateRecent"        = True
@@ -303,13 +320,16 @@ data IDEPackage     =   IDEPackage {
     ipdPackageId       ::   PackageIdentifier
 ,   ipdCabalFile       ::   FilePath
 ,   ipdDepends         ::   [Dependency]
-,   ipdModules         ::   Set ModuleName
-,   ipdMain            ::   [FilePath]
+,   ipdModules         ::   Map ModuleName BuildInfo
+,   ipdHasLibs         ::   Bool
+,   ipdTests           ::   [String]
+,   ipdMain            ::   [(FilePath, BuildInfo, Bool)]
 ,   ipdExtraSrcs       ::   Set FilePath
 ,   ipdSrcDirs         ::   [FilePath]
 ,   ipdExtensions      ::   [Extension]
 ,   ipdConfigFlags     ::   [String]
 ,   ipdBuildFlags      ::   [String]
+,   ipdTestFlags       ::   [String]
 ,   ipdHaddockFlags    ::   [String]
 ,   ipdExeFlags        ::   [String]
 ,   ipdInstallFlags    ::   [String]
@@ -393,7 +413,9 @@ data Prefs = Prefs {
     ,   docuSearchURL       ::   String
     ,   completeRestricted  ::   Bool
     ,   saveAllBeforeBuild  ::   Bool
+    ,   jumpToWarnings      ::   Bool
     ,   backgroundBuild     ::   Bool
+    ,   runUnitTests        ::   Bool
     ,   makeMode            ::   Bool
     ,   singleBuildWithoutLinking :: Bool
     ,   dontInstallLast     ::   Bool
@@ -414,8 +436,10 @@ data Prefs = Prefs {
 data SearchHint = Forward | Backward | Insert | Delete | Initial
     deriving (Eq)
 
+#ifndef LEKSAH_WITH_YI
 instance Ord Modifier
     where compare a b = compare (fromEnum a) (fromEnum b)
+#endif
 
 --
 -- | Other types
