@@ -66,7 +66,6 @@ module IDE.Debug (
 ,   debugSetPrintBindResult
 ) where
 
-import Control.Monad.Reader
 import IDE.Core.State
 import IDE.LogRef
 import Control.Exception (SomeException(..))
@@ -77,21 +76,25 @@ import IDE.Metainfo.Provider (getActivePackageDescr)
 import Distribution.Text (display)
 import IDE.Pane.Log (appendLog)
 import Data.List (isSuffixOf)
-import IDE.System.Process (interruptProcessGroup)
 import IDE.Utils.GUIUtils (getDebugToggled)
 import IDE.Package (debugStart, executeDebugCommand, tryDebug_, printBindResultFlag,
         breakOnErrorFlag, breakOnExceptionFlag, printEvldWithShowFlag)
-import IDE.Utils.Tool (ToolOutput(..), toolProcess)
+import IDE.Utils.Tool (ToolOutput(..), toolProcess, interruptProcessGroupOf)
 import IDE.Workspaces (packageTry_)
+import qualified Data.Enumerator as E
+import qualified Data.Enumerator.List as EL
+import Control.Monad.Trans.Class (MonadTrans(..))
+import Control.Monad.Trans.Reader (ask)
+import Control.Monad.IO.Class (MonadIO(..))
 
-debugCommand :: String -> ([ToolOutput] -> IDEAction) -> DebugAction
-debugCommand command handler = debugCommand' command
-    (\to -> do
-        handler to
-        triggerEventIDE VariablesChanged
-        return ())
+debugCommand :: String -> E.Iteratee ToolOutput IDEM () -> DebugAction
+debugCommand command handler = do
+    debugCommand' command $ do
+        handler
+        lift $ triggerEventIDE VariablesChanged
+        return ()
 
-debugCommand' :: String -> ([ToolOutput] -> IDEAction) -> DebugAction
+debugCommand' :: String -> E.Iteratee ToolOutput IDEM () -> DebugAction
 debugCommand' command handler = do
     ghci <- ask
     lift $ catchIDE (runDebug (executeDebugCommand command handler) ghci)
@@ -128,30 +131,29 @@ debugExecuteAndShowSelection = do
     case maybeText of
         Just text -> packageTry_ $ tryDebug_ $ do
             debugSetLiberalScope
-            debugCommand text (\to -> do
-                insertTextAfterSelection $ " " ++ buildOutputString to
-                logOutput to)
+            debugCommand text $ do
+                (out, _) <- EL.zip (EL.fold buildOutputString "") logOutput
+                lift $ insertTextAfterSelection $ " " ++ out
         Nothing   -> ideMessage Normal "Please select some text in the editor to execute"
     where
-    buildOutputString :: [ToolOutput] -> String
-    buildOutputString (ToolOutput str:[]) = str
-    buildOutputString (ToolOutput str:r)  = str ++ "\n" ++ (buildOutputString r)
-    buildOutputString (_:r)               = buildOutputString r
-    buildOutputString []                  = ""
+    buildOutputString :: String -> ToolOutput -> String
+    buildOutputString "" (ToolOutput str) = str
+    buildOutputString s  (ToolOutput str) = s ++ "\n" ++ str
+    buildOutputString s  _                = s
 
 debugSetLiberalScope :: DebugAction
 debugSetLiberalScope = do
     maybeModuleName <- lift selectedModuleName
     case maybeModuleName of
         Just moduleName -> do
-            debugCommand (":module *" ++ moduleName) (\ _ -> return ())
+            debugCommand' (":module *" ++ moduleName) logOutput
         Nothing -> do
             mbPackage <- lift getActivePackageDescr
             case mbPackage of
                 Nothing -> return ()
                 Just p -> let packageNames = map (display . modu . mdModuleId) (pdModules p)
                     in debugCommand' (foldl (\a b -> a ++ " *" ++ b) ":module + " packageNames)
-                        (\ _ -> return ())
+                        logOutput
 
 debugAbandon :: IDEAction
 debugAbandon = packageTry_ $ tryDebug_ $ debugCommand ":abandon" logOutput
@@ -178,7 +180,7 @@ debugStop :: IDEAction
 debugStop = do
     maybeDebug <- readIDE debugState
     liftIO $ case maybeDebug of
-        Just (_, ghci) -> toolProcess ghci >>= interruptProcessGroup
+        Just (_, ghci) -> toolProcess ghci >>= interruptProcessGroupOf
         Nothing -> return ()
 
 debugContinue :: IDEAction
@@ -190,14 +192,12 @@ debugContinue = packageTry_ $ do
 
 debugDeleteAllBreakpoints :: IDEAction
 debugDeleteAllBreakpoints = do
-    packageTry_ $ tryDebug_ $ debugCommand ":delete *" $ \output -> do
-        logOutput output
+    packageTry_ $ tryDebug_ $ debugCommand ":delete *" logOutput
     setBreakpointList []
 
 debugDeleteBreakpoint :: String -> LogRef -> IDEAction
 debugDeleteBreakpoint indexString lr = do
-    packageTry_ $ tryDebug_ $ debugCommand (":delete " ++ indexString) $ \output -> do
-        logOutput output
+    packageTry_ $ tryDebug_ $ debugCommand (":delete " ++ indexString) logOutput
     bl <- readIDE breakpointRefs
     setBreakpointList $ filter (/= lr) bl
     ideR <- ask
@@ -269,10 +269,10 @@ debugTrace = packageTry_ $ do
     rootPath <- lift $ activeProjectDir
     tryDebug_ $ do
         (debugPackage, _) <- ask
-        debugCommand ":trace" (\to -> do
-            logOutputForLiveContext debugPackage to
-            triggerEventIDE TraceChanged
-            return ())
+        debugCommand ":trace" $ do
+            logOutputForLiveContext debugPackage
+            lift $ triggerEventIDE TraceChanged
+            return ()
 
 debugTraceExpression :: IDEAction
 debugTraceExpression = do
@@ -285,11 +285,11 @@ debugTraceExpr :: Maybe String -> DebugAction
 debugTraceExpr maybeText = do
     (debugPackage, _) <- ask
     case maybeText of
-        Just text -> debugCommand (":trace " ++ text) (\to -> do
-            rootPath <- activeProjectDir
-            logOutputForLiveContext debugPackage to
-            triggerEventIDE TraceChanged
-            return ())
+        Just text -> debugCommand (":trace " ++ text) $ do
+--            rootPath <- activeProjectDir
+            logOutputForLiveContext debugPackage
+            lift $ triggerEventIDE TraceChanged
+            return ()
         Nothing   -> lift $ ideMessage Normal "Please select an expression in the editor"
 
 
@@ -319,7 +319,7 @@ debugShowModules = packageTry_ $ tryDebug_ $ debugCommand ":show modules" $
                             -> appendLog log (line ++ "\n") LogTag
             ToolOutput line -> appendLog log (line ++ "\n") InfoTag
             ToolError  line -> appendLog log (line ++ "\n") ErrorTag
-            ToolPrompt      -> defaultLineLogger' log output
+            ToolPrompt _    -> defaultLineLogger' log output
             ToolExit _      -> appendLog log "X--X--X ghci process exited unexpectedly X--X--X" FrameTag
         return ()
 
@@ -367,7 +367,7 @@ debugSetBreakpoint = do
             case maybeText of
                 Just text -> packageTry_ $ tryDebug_ $ do
                     (debugPackage, _) <- ask
-                    debugCommand (":module *" ++ moduleName) logOutput
+                    debugCommand' (":module *" ++ moduleName) logOutput
                     debugCommand (":break " ++ text) (logOutputForSetBreakpoint debugPackage)
                 Nothing   -> do
                     maybeLocation <- selectedLocation
