@@ -48,16 +48,28 @@ import qualified Distribution.ModuleName as D (ModuleName(..))
 import qualified Text.ParserCombinators.Parsec.Token as P
        (operator, dot, identifier, symbol, lexeme, whiteSpace,
         makeTokenParser)
-import Control.Monad.Trans (liftIO)
 import Distribution.PackageDescription.Parse
-       (writePackageDescription, readPackageDescription)
+       (readPackageDescription)
 import Distribution.Verbosity (normal)
 import IDE.Pane.PackageEditor (hasConfigs)
 import Distribution.Package
-import Distribution.Version (VersionRange(..))
-import Distribution.PackageDescription (buildDepends)
+import Distribution.Version (anyVersion)
+import Distribution.PackageDescription
+       (CondTree(..), condExecutables, condLibrary, packageDescription,
+        buildDepends)
 import Distribution.PackageDescription.Configuration
        (flattenPackageDescription)
+import IDE.BufferMode (editInsertCode)
+import Control.Monad.IO.Class (MonadIO(..))
+#if MIN_VERSION_Cabal(1,10,0)
+import Distribution.PackageDescription.PrettyPrintCopied
+       (writeGenericPackageDescription)
+#else
+import Distribution.PackageDescription.Parse
+       (writePackageDescription)
+import Distribution.PackageDescription
+       (CondTree(..))
+#endif
 
 -- | Add all imports which gave error messages ...
 resolveErrors :: IDEAction
@@ -133,16 +145,30 @@ addPackage error = do
         Nothing -> return False
         Just (HiddenModuleResult mod pack) -> do
             let idePackage = logRefPackage error
-            package <- liftIO $ readPackageDescription normal (ipdCabalFile $ idePackage)
-            if hasConfigs package
+            gpd <- liftIO $ readPackageDescription normal (ipdCabalFile $ idePackage)
+            ideMessage Normal $ "addPackage " ++ (display $ pkgName pack)
+#if MIN_VERSION_Cabal(1,10,0)
+            liftIO $ writeGenericPackageDescription (ipdCabalFile $ idePackage)
+                gpd { condLibrary     = addDepToLib (packageName pack)  (condLibrary gpd),
+                      condExecutables = map (addDepToExe (packageName pack))
+                                            (condExecutables gpd)}
+            return True
+#else
+            if hasConfigs gpd
                 then return False
                 else do
-                    let flat = flattenPackageDescription package
-                    ideMessage Normal $ "addPackage " ++ (display $ pkgName pack)
+                    let flat = flattenPackageDescription gpd
                     liftIO $ writePackageDescription (ipdCabalFile $ idePackage)
                         flat { buildDepends =
-                            Dependency (pkgName pack) AnyVersion : buildDepends flat}
+                            Dependency (pkgName pack) anyVersion : buildDepends flat}
                     return True
+#endif
+  where
+    addDepToLib n Nothing = Nothing
+    addDepToLib n (Just cn@CondNode{condTreeConstraints = deps}) =
+        Just (cn{condTreeConstraints = (Dependency n anyVersion) : deps})
+    addDepToExe n (str,cn@CondNode{condTreeConstraints = deps}) =
+        (str,cn{condTreeConstraints = (Dependency n anyVersion) : deps})
 
 getScopeForActiveBuffer :: IDEM (Maybe (GenScope, GenScope))
 getScopeForActiveBuffer = do
@@ -174,7 +200,7 @@ addImport' nis filePath descr descrList continuation =  do
                                                 lastLine = foldr max 0 (map (locationELine . importLoc) imports)
                                             in do
                                                 i1 <- getIterAtLine gtkbuf lastLine
-                                                insert gtkbuf i1 newLine
+                                                editInsertCode gtkbuf i1 newLine
                                                 fileSave False
                                                 setModified gtkbuf True
                                                 continuation (True,(descr : descrList))
@@ -188,7 +214,7 @@ addImport' nis filePath descr descrList continuation =  do
                                                     i1 <- getIterAtLine gtkbuf (lineStart - 1)
                                                     i2 <- getIterAtLine gtkbuf (lineEnd)
                                                     delete gtkbuf i1 i2
-                                                    insert gtkbuf i1 newLine
+                                                    editInsertCode gtkbuf i1 newLine
                                                     fileSave False
                                                     setModified gtkbuf True
                                                     continuation (True,(descr : descrList))
@@ -196,7 +222,7 @@ addImport' nis filePath descr descrList continuation =  do
                                             let newLine  =  prettyPrint (newImpDecl mod) ++ "\n"
                                             in do
                                                 i1 <- getIterAtLine gtkbuf lastLine
-                                                insert gtkbuf i1 newLine
+                                                editInsertCode gtkbuf i1 newLine
                                                 fileSave False
                                                 setModified gtkbuf True
                                                 continuation (True,(descr : descrList))
