@@ -15,7 +15,6 @@
 --
 ---------------------------------------------------------------------------------
 
-
 module IDE.Package (
     packageConfig
 ,   packageConfig'
@@ -25,14 +24,16 @@ module IDE.Package (
 ,   packageClean
 ,   packageClean'
 ,   packageCopy
+,   packageCopy'
 ,   packageRun
 ,   activatePackage
 ,   deactivatePackage
 
-,   packageInstall
-,   packageInstall'
+,   packageInstallDependencies
 ,   packageRegister
+,   packageRegister'
 ,   packageTest
+,   packageTest'
 ,   packageSdist
 ,   packageOpenDoc
 
@@ -43,6 +44,7 @@ module IDE.Package (
 ,   delModuleFromPackageDescr
 
 ,   backgroundBuildToggled
+,   runUnitTestsToggled
 ,   makeModeToggled
 
 ,   debugStart
@@ -53,15 +55,17 @@ module IDE.Package (
 ,   printEvldWithShowFlag
 ,   tryDebug
 ,   tryDebug_
+,   tryDebugQuiet
+,   tryDebugQuiet_
 ,   executeDebugCommand
 
 ,   choosePackageFile
 
 ,   idePackageFromPath
+
 ) where
 
 import Graphics.UI.Gtk
-import Control.Monad.Reader
 import Distribution.Package hiding (depends,packageId)
 import Distribution.PackageDescription
 import Distribution.PackageDescription.Parse
@@ -89,20 +93,35 @@ import Data.List (isInfixOf, nub, foldl', delete)
 import qualified System.IO.UTF8 as UTF8  (readFile)
 import IDE.Utils.Tool (ToolOutput(..), runTool, newGhci, ToolState(..))
 import qualified Data.Set as  Set (fromList)
-import qualified Data.Map as  Map (empty)
+import qualified Data.Map as  Map (empty, fromList)
 import System.Exit (ExitCode(..))
 import Control.Applicative ((<$>))
-import IDE.System.Process (getProcessExitCode, interruptProcessGroup)
-import IDE.Utils.Tool (executeGhciCommand)
+import IDE.Utils.Tool (executeGhciCommand, getProcessExitCode, interruptProcessGroupOf)
+import qualified Data.Enumerator as E (run_, Iteratee(..), last)
+import qualified Data.Enumerator.List as EL (foldM, zip3, zip)
+import Data.Enumerator (($$))
+import Control.Monad.Trans.Reader (ask)
+import Control.Monad.IO.Class (MonadIO(..))
+import Control.Monad.Trans.Class (lift)
+import Control.Monad (when, unless, liftM)
+#if MIN_VERSION_Cabal(1,10,0)
+import Distribution.PackageDescription.PrettyPrintCopied
+       (writeGenericPackageDescription)
+#endif
+import Debug.Trace (trace)
+
+moduleInfo :: (a -> BuildInfo) -> (a -> [ModuleName]) -> a -> [(ModuleName, BuildInfo)]
+moduleInfo bi mods a = map (\m -> (m, buildInfo)) $ mods a
+    where buildInfo = bi a
 
 #if MIN_VERSION_Cabal(1,8,0)
 myLibModules pd = case library pd of
                     Nothing -> []
-                    Just l -> libModules l
-myExeModules pd = concatMap exeModules (executables pd)
+                    Just l -> moduleInfo libBuildInfo libModules l
+myExeModules pd = concatMap (moduleInfo buildInfo exeModules) (executables pd)
 #else
-myLibModules pd = libModules pd
-myExeModules pd = exeModules pd
+myLibModules pd = moduleInfo libModules libBuildInfo pd
+myExeModules pd = moduleInfo exeModules buildInfo pd
 #endif
 
 
@@ -166,69 +185,54 @@ packageConfig'  :: IDEPackage -> (Bool -> IDEAction) -> IDEAction
 packageConfig' package continuation = do
     let dir = dropFileName (ipdCabalFile package)
     runExternalTool "Configuring" "cabal" (["configure"]
-                                    ++ (ipdConfigFlags package)) (Just dir) $ \output -> do
-        logOutput output
-        mbPack <- idePackageFromPath (ipdCabalFile package)
-        case mbPack of
-            Just pack -> do
-                changeWorkspacePackage pack
-                modifyIDE_ (\ide -> ide{bufferProjCache = Map.empty})
-                mbActivePack <- readIDE activePack
-                case mbActivePack of
-                    Just activePack | ipdCabalFile package == ipdCabalFile activePack ->
-                        modifyIDE_ (\ide -> ide{activePack = Just pack})
-                    _ -> return ()
-                triggerEventIDE UpdateWorkspaceInfo
-                triggerEventIDE (WorkspaceChanged False True)
-                continuation (last output == ToolExit ExitSuccess)
-                return ()
-            Nothing -> do
-                ideMessage Normal "Can't read package file"
-                continuation False
-                return()
+                                    ++ (ipdConfigFlags package)) (Just dir) $ do
+        (mbLastOutput, _) <- EL.zip E.last logOutput
+        lift $ do
+            mbPack <- idePackageFromPath (ipdCabalFile package)
+            case mbPack of
+                Just pack -> do
+                    changePackage pack
+                    triggerEventIDE (WorkspaceChanged False True)
+                    continuation (mbLastOutput == Just (ToolExit ExitSuccess))
+                    return ()
+                Nothing -> do
+                    ideMessage Normal "Can't read package file"
+                    continuation False
+                    return()
 
-changeWorkspacePackage :: IDEPackage -> IDEAction
-changeWorkspacePackage ideP@IDEPackage{ipdCabalFile = file} = do
-    oldWorkspace <- readIDE workspace
-    case oldWorkspace of
-        Nothing -> return ()
-        Just ws ->
-            let ps = map exchange (wsPackages ws)
-            in modifyIDE_ (\ide -> ide{workspace = Just  ws {wsPackages = ps}})
-    where
-        exchange p | ipdCabalFile p == file = ideP
-                   | otherwise              = p
-
-runCabalBuild :: Bool -> Bool -> IDEPackage -> Bool -> (Bool -> IDEAction) -> IDEAction
-runCabalBuild backgroundBuild withoutLinking package shallConfigure continuation = do
-    prefs   <- readIDE prefs
+runCabalBuild :: Bool -> Bool -> Bool -> IDEPackage -> Bool -> (Bool -> IDEAction) -> IDEAction
+runCabalBuild backgroundBuild jumpToWarnings withoutLinking package shallConfigure continuation = do
+    prefs <- readIDE prefs
     let dir =  dropFileName (ipdCabalFile package)
     let args = (["build"] ++
                 if backgroundBuild && withoutLinking
                     then ["--with-ld=false"]
                     else []
                         ++ ipdBuildFlags package)
-    runExternalTool "Building" "cabal" args (Just dir) $ \output -> do
-        logOutputForBuild package backgroundBuild output
-        errs <- readIDE errorRefs
-        if shallConfigure && isConfigError output
-            then
-                packageConfig' package (\ b ->
-                    when b $ runCabalBuild backgroundBuild withoutLinking package False continuation)
-            else do
-                continuation (last output == ToolExit ExitSuccess)
-                return ()
+    runExternalTool "Building" "cabal" args (Just dir) $ do
+        (mbLastOutput, isConfigErr, _) <- EL.zip3 E.last isConfigError $
+            logOutputForBuild package backgroundBuild jumpToWarnings
+        lift $ do
+            errs <- readIDE errorRefs
+            if shallConfigure && isConfigErr
+                then
+                    packageConfig' package (\ b ->
+                        when b $ runCabalBuild backgroundBuild jumpToWarnings withoutLinking package False continuation)
+                else do
+                    continuation (mbLastOutput == Just (ToolExit ExitSuccess))
+                    return ()
 
-isConfigError :: [ToolOutput] -> Bool
-isConfigError = or . (map isCErr)
+isConfigError :: Monad m => E.Iteratee ToolOutput m Bool
+isConfigError = EL.foldM (\a b -> return $ a || isCErr b) False
     where
-    isCErr (ToolError str) = str1 `isInfixOf` str || str2 `isInfixOf` str
+    isCErr (ToolError str) = str1 `isInfixOf` str || str2 `isInfixOf` str || str3 `isInfixOf` str
     isCErr _ = False
     str1 = "Run the 'configure' command first"
     str2 = "please re-configure"
+    str3 = "cannot satisfy -package-id"
 
-buildPackage :: Bool -> Bool -> IDEPackage -> (Bool -> IDEAction) -> IDEAction
-buildPackage backgroundBuild withoutLinking package continuation = catchIDE (do
+buildPackage :: Bool -> Bool -> Bool -> IDEPackage -> (Bool -> IDEAction) -> IDEAction
+buildPackage backgroundBuild jumpToWarnings withoutLinking package continuation = catchIDE (do
     ideR      <- ask
     prefs     <- readIDE prefs
     maybeDebug <- readIDE debugState
@@ -241,19 +245,19 @@ buildPackage backgroundBuild withoutLinking package continuation = catchIDE (do
                     when (not backgroundBuild) $ liftIO $ do
                         timeoutAddFull (do
                             reflectIDE (do
-                                buildPackage backgroundBuild withoutLinking
+                                buildPackage backgroundBuild jumpToWarnings withoutLinking
                                                 package continuation
                                 return False) ideR
                             return False) priorityDefaultIdle 1000
                         return ()
-                else runCabalBuild backgroundBuild withoutLinking package True continuation
+                else runCabalBuild backgroundBuild jumpToWarnings withoutLinking package True continuation
         Just debug@(_, ghci) -> do
             -- TODO check debug package matches active package
             ready <- liftIO $ isEmptyMVar (currentToolCommand ghci)
             when ready $ do
                 let dir = dropFileName (ipdCabalFile package)
                 when (saveAllBeforeBuild prefs) (do fileSaveAll belongsToWorkspace; return ())
-                runDebug (executeDebugCommand ":reload" (logOutputForBuild package backgroundBuild)) debug
+                runDebug (executeDebugCommand ":reload" (logOutputForBuild package backgroundBuild jumpToWarnings)) debug
     )
     (\(e :: SomeException) -> sysMessage Normal (show e))
 
@@ -272,12 +276,11 @@ packageClean = do
     lift $ packageClean' package (\ _ -> return ())
 
 packageClean' :: IDEPackage -> (Bool -> IDEAction) -> IDEAction
-packageClean' package continuation =
+packageClean' package continuation = do
     let dir = dropFileName (ipdCabalFile package)
-    in runExternalTool "Cleaning" "cabal" ["clean"] (Just dir)
-        (\ output -> do
-            logOutput output
-            continuation (last output == ToolExit ExitSuccess))
+    runExternalTool "Cleaning" "cabal" ["clean"] (Just dir) $ do
+        (mbLastOutput, _) <- EL.zip E.last logOutput
+        lift $ continuation (mbLastOutput == Just (ToolExit ExitSuccess))
 
 packageCopy :: PackageAction
 packageCopy = do
@@ -293,6 +296,25 @@ packageCopy = do
                            ++ ["--destdir=" ++ fp]) (Just dir) logOutput)
         (\(e :: SomeException) -> putStrLn (show e))
 
+packageInstallDependencies :: PackageAction
+packageInstallDependencies = do
+    package <- ask
+    lift $ catchIDE (do
+        let dir = dropFileName (ipdCabalFile package)
+        runExternalTool "Installing" "cabal" (["install","--only-dependencies"]
+            ++ (ipdInstallFlags package)) (Just dir) logOutput)
+        (\(e :: SomeException) -> putStrLn (show e))
+
+packageCopy' :: IDEPackage -> (Bool -> IDEAction) -> IDEAction
+packageCopy' package continuation = do
+    catchIDE (do
+        let dir = dropFileName (ipdCabalFile package)
+        runExternalTool "Copying" "cabal" (["copy"]
+            ++ (ipdInstallFlags package)) (Just dir) $ do
+                (mbLastOutput, _) <- EL.zip E.last logOutput
+                lift $ continuation (mbLastOutput == Just (ToolExit ExitSuccess)))
+        (\(e :: SomeException) -> putStrLn (show e))
+
 packageRun :: PackageAction
 packageRun = do
     package <- ask
@@ -322,36 +344,39 @@ packageRun = do
                         return ())
         (\(e :: SomeException) -> putStrLn (show e))
 
-packageInstall :: PackageAction
-packageInstall = do
-    package <- ask
-    lift $ packageInstall' package (\ _ -> return ())
-
-packageInstall' :: IDEPackage -> (Bool -> IDEAction) -> IDEAction
-packageInstall' package continuation = catchIDE (do
-   let dir = dropFileName (ipdCabalFile package)
-   runExternalTool "Installing" "runhaskell" (["Setup", "install"]
-                    ++ (ipdInstallFlags package)) (Just dir) (\ output -> do
-                        logOutput output
-                        continuation (last output == ToolExit ExitSuccess)))
-        (\(e :: SomeException) -> putStrLn (show e))
-
 packageRegister :: PackageAction
 packageRegister = do
     package <- ask
-    lift $ catchIDE (do
-        let dir = dropFileName (ipdCabalFile package)
-        runExternalTool "Registering" "cabal" (["register"]
-                        ++ (ipdRegisterFlags package)) (Just dir) logOutput)
-        (\(e :: SomeException) -> putStrLn (show e))
+    lift $ packageRegister' package (\ _ -> return ())
+
+packageRegister' :: IDEPackage -> (Bool -> IDEAction) -> IDEAction
+packageRegister' package continuation =
+    if ipdHasLibs package
+        then catchIDE (do
+            let dir = dropFileName (ipdCabalFile package)
+            runExternalTool "Registering" "cabal" (["register"]
+                ++ (ipdRegisterFlags package)) (Just dir) $ do
+                    (mbLastOutput, _) <- EL.zip E.last logOutput
+                    lift $ continuation (mbLastOutput == Just (ToolExit ExitSuccess)))
+            (\(e :: SomeException) -> putStrLn (show e))
+        else continuation True
 
 packageTest :: PackageAction
 packageTest = do
     package <- ask
-    lift $ catchIDE (do
-        let dir = dropFileName (ipdCabalFile package)
-        runExternalTool "Testing" "cabal" (["test"]) (Just dir) logOutput)
-        (\(e :: SomeException) -> putStrLn (show e))
+    lift $ packageTest' package (\ _ -> return ())
+
+packageTest' :: IDEPackage -> (Bool -> IDEAction) -> IDEAction
+packageTest' package continuation =
+    if "--enable-tests" `elem` ipdConfigFlags package
+        then catchIDE (do
+            let dir = dropFileName (ipdCabalFile package)
+            runExternalTool "Testing" "cabal" (["test"]
+                ++ (ipdTestFlags package)) (Just dir) $ do
+                    (mbLastOutput, _) <- EL.zip E.last logOutput
+                    lift $ continuation (mbLastOutput == Just (ToolExit ExitSuccess)))
+            (\(e :: SomeException) -> putStrLn (show e))
+        else continuation True
 
 packageSdist :: PackageAction
 packageSdist = do
@@ -377,18 +402,17 @@ packageOpenDoc = do
         runExternalTool "Opening Documentation" (browser prefs) [path] (Just dir) logOutput)
         (\(e :: SomeException) -> putStrLn (show e))
 
-runExternalTool :: String -> FilePath -> [String] -> Maybe FilePath -> ([ToolOutput] -> IDEAction) -> IDEAction
+runExternalTool :: String -> FilePath -> [String] -> Maybe FilePath -> E.Iteratee ToolOutput IDEM () -> IDEAction
 runExternalTool description executable args mbDir handleOutput = do
         prefs          <- readIDE prefs
         alreadyRunning <- isRunning
         unless alreadyRunning $ do
             when (saveAllBeforeBuild prefs) (do fileSaveAll belongsToWorkspace; return ())
             triggerEventIDE (StatusbarChanged [CompartmentState description, CompartmentBuild True])
-            reifyIDE (\ideR -> forkIO $ do
+            reifyIDE $ \ideR -> forkIO $ do
                 (output, pid) <- runTool executable args mbDir
-                reflectIDE (do
-                    modifyIDE_ (\ide -> ide{runningTool = Just pid})
-                    handleOutput output) ideR)
+                reflectIDE (modifyIDE_ (\ide -> ide{runningTool = Just pid})) ideR
+                E.run_ $ output $$ (reflectIDEI handleOutput ideR)
             return ()
 
 
@@ -408,7 +432,7 @@ interruptBuild :: IDEAction
 interruptBuild = do
     maybeProcess <- readIDE runningTool
     liftIO $ case maybeProcess of
-        Just h -> interruptProcessGroup h
+        Just h -> interruptProcessGroupOf h
         _ -> return ()
 
 -- ---------------------------------------------------------------------
@@ -432,12 +456,12 @@ getPackageDescriptionAndPath = do
                         return Nothing))
 
 getEmptyModuleTemplate :: PackageDescription -> String -> IO String
-getEmptyModuleTemplate pd modName = getModuleTemplate pd modName "" ""
+getEmptyModuleTemplate pd modName = getModuleTemplate "module" pd modName "" ""
 
-getModuleTemplate :: PackageDescription -> String -> String -> String -> IO String
-getModuleTemplate pd modName exports body = catch (do
+getModuleTemplate :: String -> PackageDescription -> String -> String -> String -> IO String
+getModuleTemplate template pd modName exports body = catch (do
     dataDir  <- getDataDir
-    filePath <- getConfigFilePathForLoad standardModuleTemplateFilename Nothing dataDir
+    filePath <- getConfigFilePathForLoad (template ++ leksahTemplateFileExtension) Nothing dataDir
     template <- UTF8.readFile filePath
     return (foldl' (\ a (from, to) -> replace from to a) template
         [   ("@License@"      , (show . license) pd)
@@ -450,6 +474,95 @@ getModuleTemplate pd modName exports body = catch (do
         ,   ("@ModuleBody@"   , body)]))
                     (\ (e :: SomeException) -> sysMessage Normal ("Couldn't read template file: " ++ show e) >> return "")
 
+#if MIN_VERSION_Cabal(1,10,0)
+addModuleToPackageDescr :: ModuleName -> Bool -> PackageAction
+addModuleToPackageDescr moduleName isExposed = do
+    p    <- ask
+    lift $ reifyIDE (\ideR -> catch (do
+        gpd <- readPackageDescription normal (ipdCabalFile p)
+        let npd = if isExposed && isJust (condLibrary gpd)
+                then gpd{
+                    condLibrary = Just (addModToLib moduleName
+                                                (fromJust (condLibrary gpd))),
+                    condExecutables = map (addModToBuildInfoExe moduleName)
+                                            (condExecutables gpd)}
+                else gpd{
+                    condLibrary = case condLibrary gpd of
+                                    Nothing -> Nothing
+                                    Just lib -> Just (addModToBuildInfoLib moduleName
+                                                       (fromJust (condLibrary gpd))),
+                    condExecutables = map (addModToBuildInfoExe moduleName)
+                                                (condExecutables gpd)}
+        writeGenericPackageDescription (ipdCabalFile p) npd)
+           (\(e :: SomeException) -> do
+            reflectIDE (ideMessage Normal ("Can't update package " ++ show e)) ideR
+            return ()))
+
+addModToLib :: ModuleName -> CondTree ConfVar [Dependency] Library ->
+    CondTree ConfVar [Dependency] Library
+addModToLib modName ct@CondNode{condTreeData = lib} =
+    ct{condTreeData = lib{exposedModules = modName : exposedModules lib}}
+
+addModToBuildInfoLib :: ModuleName -> CondTree ConfVar [Dependency] Library ->
+    CondTree ConfVar [Dependency] Library
+addModToBuildInfoLib modName ct@CondNode{condTreeData = lib} =
+    ct{condTreeData = lib{libBuildInfo = (libBuildInfo lib){otherModules = modName
+        : otherModules (libBuildInfo lib)}}}
+
+addModToBuildInfoExe :: ModuleName -> (String, CondTree ConfVar [Dependency] Executable) ->
+    (String, CondTree ConfVar [Dependency] Executable)
+addModToBuildInfoExe modName (str,ct@CondNode{condTreeData = exe}) =
+    (str, ct{condTreeData = exe{buildInfo = (buildInfo exe){otherModules = modName
+        : otherModules (buildInfo exe)}}})
+
+--------------------------------------------------------------------------
+delModuleFromPackageDescr :: ModuleName -> PackageAction
+delModuleFromPackageDescr moduleName = trace ("addModule " ++ show moduleName) $ do
+    p    <- ask
+    lift $ reifyIDE (\ideR -> catch (do
+        gpd <- readPackageDescription normal (ipdCabalFile p)
+        let isExposedAndJust = isExposedModule moduleName (condLibrary gpd)
+        let npd = if isExposedAndJust
+                then gpd{
+                    condLibrary = Just (delModFromLib moduleName
+                                                (fromJust (condLibrary gpd))),
+                    condExecutables = map (delModFromBuildInfoExe moduleName)
+                                            (condExecutables gpd)}
+                else gpd{
+                    condLibrary = case condLibrary gpd of
+                                    Nothing -> Nothing
+                                    Just lib -> Just (delModFromBuildInfoLib moduleName
+                                                       (fromJust (condLibrary gpd))),
+                    condExecutables = map (delModFromBuildInfoExe moduleName)
+                                                (condExecutables gpd)}
+        writeGenericPackageDescription (ipdCabalFile p) npd)
+           (\(e :: SomeException) -> do
+            reflectIDE (ideMessage Normal ("Can't update package " ++ show e)) ideR
+            return ()))
+
+delModFromLib :: ModuleName -> CondTree ConfVar [Dependency] Library ->
+    CondTree ConfVar [Dependency] Library
+delModFromLib modName ct@CondNode{condTreeData = lib} =
+    ct{condTreeData = lib{exposedModules = delete modName (exposedModules lib)}}
+
+delModFromBuildInfoLib :: ModuleName -> CondTree ConfVar [Dependency] Library ->
+    CondTree ConfVar [Dependency] Library
+delModFromBuildInfoLib modName ct@CondNode{condTreeData = lib} =
+    ct{condTreeData = lib{libBuildInfo = (libBuildInfo lib){otherModules =
+        delete modName (otherModules (libBuildInfo lib))}}}
+
+delModFromBuildInfoExe :: ModuleName -> (String, CondTree ConfVar [Dependency] Executable) ->
+    (String, CondTree ConfVar [Dependency] Executable)
+delModFromBuildInfoExe modName (str,ct@CondNode{condTreeData = exe}) =
+    (str, ct{condTreeData = exe{buildInfo = (buildInfo exe){otherModules =
+        delete modName (otherModules (buildInfo exe))}}})
+
+isExposedModule :: ModuleName -> Maybe (CondTree ConfVar [Dependency] Library)  -> Bool
+isExposedModule mn Nothing                             = False
+isExposedModule mn (Just CondNode{condTreeData = lib}) = elem mn (exposedModules lib)
+
+#else
+-- Old version to support older Cabal
 addModuleToPackageDescr :: ModuleName -> Bool -> PackageAction
 addModuleToPackageDescr moduleName isExposed = do
     p    <- ask
@@ -479,7 +592,7 @@ addModuleToPackageDescr moduleName isExposed = do
     addModToBuildInfo :: BuildInfo -> ModuleName -> BuildInfo
     addModToBuildInfo bi mn = bi {otherModules = mn : otherModules bi}
 
---------------------------------------------------------------------------
+-- Old version to support older Cabal
 delModuleFromPackageDescr :: ModuleName -> PackageAction
 delModuleFromPackageDescr moduleName = do
     p    <- ask
@@ -510,21 +623,24 @@ delModuleFromPackageDescr moduleName = do
     delModFromBuildInfo :: BuildInfo -> ModuleName -> BuildInfo
     delModFromBuildInfo bi mn = bi {otherModules = delete mn (otherModules bi)}
 
-
+-- Old version to support older Cabal
 isExposedModule :: PackageDescription -> ModuleName -> Bool
 isExposedModule pd mn = do
     if isJust (library pd)
         then elem mn (exposedModules (fromJust $ library pd))
         else False
-
---------------------------------------------------------------------------
-
+#endif
 
 backgroundBuildToggled :: IDEAction
 backgroundBuildToggled = do
     toggled <- getBackgroundBuildToggled
     modifyIDE_ (\ide -> ide{prefs = (prefs ide){backgroundBuild= toggled}})
 
+runUnitTestsToggled :: IDEAction
+runUnitTestsToggled = do
+    toggled <- getRunUnitTests
+    modifyIDE_ (\ide -> ide{prefs = (prefs ide){runUnitTests= toggled}})
+
 makeModeToggled :: IDEAction
 makeModeToggled = do
     toggled <- getMakeModeToggled
@@ -566,14 +682,14 @@ debugStart = do
         case maybeDebug of
             Nothing -> do
                 ghci <- reifyIDE $ \ideR -> newGhci (ipdBuildFlags package) (interactiveFlags prefs')
-                    $ \output -> reflectIDE (logOutputForBuild package True output) ideR
+                    $ reflectIDEI (logOutputForBuild package True False) ideR
                 modifyIDE_ (\ide -> ide {debugState = Just (package, ghci)})
                 triggerEventIDE (Sensitivity [(SensitivityInterpreting, True)])
                 setDebugToggled True
                 -- Fork a thread to wait for the output from the process to close
                 liftIO $ forkIO $ do
                     readMVar (outputClosed ghci)
-                    reflectIDE (do
+                    postGUISync $ reflectIDE (do
                         setDebugToggled False
                         modifyIDE_ (\ide -> ide {debugState = Nothing})
                         triggerEventIDE (Sensitivity [(SensitivityInterpreting, False)])
@@ -587,7 +703,7 @@ debugStart = do
                             -- Lets build to make sure the binaries are up to date
                             mbPackage   <- readIDE activePack
                             case mbPackage of
-                                Just package -> runCabalBuild True True package True (\ _ -> return ())
+                                Just package -> runCabalBuild True False True package True (\ _ -> return ())
                                 Nothing -> return ()) ideRef
                 return ()
             _ -> do
@@ -625,18 +741,29 @@ tryDebug f = do
 tryDebug_ :: DebugM a -> PackageAction
 tryDebug_ f = tryDebug f >> return ()
 
-executeDebugCommand :: String -> ([ToolOutput] -> IDEAction) -> DebugAction
+tryDebugQuiet :: DebugM a -> PackageM (Maybe a)
+tryDebugQuiet f = do
+    maybeDebug <- lift $ readIDE debugState
+    case maybeDebug of
+        Just debug -> do
+            -- TODO check debug package matches active package
+            liftM Just $ lift $ runDebug f debug
+        _ -> do
+            return Nothing
+
+tryDebugQuiet_ :: DebugM a -> PackageAction
+tryDebugQuiet_ f = tryDebugQuiet f >> return ()
+
+executeDebugCommand :: String -> (E.Iteratee ToolOutput IDEM ()) -> DebugAction
 executeDebugCommand command handler = do
     (_, ghci) <- ask
     lift $ do
         triggerEventIDE (StatusbarChanged [CompartmentState command, CompartmentBuild True])
         reifyIDE $ \ideR -> do
-            executeGhciCommand ghci command $ \output ->
-                reflectIDE (do
-                    handler output
-                    triggerEventIDE (StatusbarChanged [CompartmentState "", CompartmentBuild False])
-                    return ()
-                    ) ideR
+            executeGhciCommand ghci command $ do
+                reflectIDEI handler ideR
+                liftIO $ postGUISync $ reflectIDE (triggerEventIDE (StatusbarChanged [CompartmentState "", CompartmentBuild False])) ideR
+                return ()
 
 allBuildInfo' :: PackageDescription -> [BuildInfo]
 #if MIN_VERSION_Cabal(1,10,0)
@@ -658,28 +785,38 @@ idePackageFromPath filePath = do
     case mbPackageD of
         Nothing       -> return Nothing
         Just packageD -> do
-            let modules    = Set.fromList $ myLibModules packageD ++ myExeModules packageD
-            let mainFiles  = map modulePath (executables packageD)
-            let files      = Set.fromList $ extraSrcFiles packageD ++ map modulePath (executables packageD)
+            let modules    = Map.fromList $ myLibModules packageD ++ myExeModules packageD
+            let mainFiles  = [ (modulePath exe, buildInfo exe, False) | exe <- executables packageD ]
+#if MIN_VERSION_Cabal(1,10,0)
+                             ++ [ (f, bi, True) | TestSuite _ (TestSuiteExeV10 _ f) bi _ <- testSuites packageD ]
+#endif
+            let files      = Set.fromList $ extraSrcFiles packageD
             let srcDirs = case (nub $ concatMap hsSourceDirs (allBuildInfo' packageD)) of
                                 [] -> [".","src"]
                                 l -> l
 #if MIN_VERSION_Cabal(1,10,0)
             let exts       = nub $ concatMap oldExtensions (allBuildInfo' packageD)
+            let tests      = [ testName t | t <- testSuites packageD
+                                          , buildable (testBuildInfo t) ]
 #else
             let exts       = nub $ concatMap extensions (allBuildInfo' packageD)
+            let tests      = []
 #endif
+
             let packp      = IDEPackage {
                 ipdPackageId = package packageD,
                 ipdCabalFile = filePath,
                 ipdDepends = buildDepends packageD,
                 ipdModules = modules,
+                ipdHasLibs = hasLibs packageD,
+                ipdTests   = tests,
                 ipdMain    = mainFiles,
                 ipdExtraSrcs =  files,
                 ipdSrcDirs = srcDirs,
                 ipdExtensions =  exts,
-                ipdConfigFlags = ["--user"],
+                ipdConfigFlags = ["--user", "--enable-tests"],
                 ipdBuildFlags = [],
+                ipdTestFlags = [],
                 ipdHaddockFlags = [],
                 ipdExeFlags = [],
                 ipdInstallFlags = [],
