@@ -1,5 +1,5 @@
-{-# OPTIONS_GHC -XRecordWildCards -XTypeSynonymInstances -XMultiParamTypeClasses
-    -XDeriveDataTypeable #-}
+{-# LANGUAGE FlexibleInstances, RecordWildCards, TypeSynonymInstances,
+             MultiParamTypeClasses, DeriveDataTypeable #-}
 -----------------------------------------------------------------------------
 --
 -- Module      :  IDE.Pane.Breakpoints
@@ -24,7 +24,6 @@ module IDE.Pane.Breakpoints (
 import Graphics.UI.Gtk
 import Data.Typeable (Typeable(..))
 import IDE.Core.State
-import Control.Monad.Reader
 import Graphics.UI.Gtk.Gdk.Events (Event(..))
 import Graphics.UI.Gtk.General.Enums
     (Click(..), MouseButton(..))
@@ -34,6 +33,7 @@ import IDE.Debug
      debugDeleteAllBreakpoints)
 import IDE.LogRef (showSourceSpan)
 import Data.List (elemIndex)
+import Control.Monad.IO.Class (MonadIO(..))
 
 
 -- | A breakpoints pane description
