@@ -1,5 +1,6 @@
-{-# OPTIONS_GHC -XDeriveDataTypeable -XMultiParamTypeClasses
-    -XScopedTypeVariables -XTypeSynonymInstances #-}
+{-# LANGUAGE FlexibleInstances, DeriveDataTypeable, MultiParamTypeClasses,
+             ScopedTypeVariables, TypeSynonymInstances #-}
+{-# OPTIONS_GHC -fwarn-unused-imports #-}
 -----------------------------------------------------------------------------
 --
 -- Module      :  IDE.Pane.Info
@@ -24,16 +25,18 @@ module IDE.Pane.Info (
 
 import Graphics.UI.Gtk hiding (afterToggleOverwrite)
 import Control.Monad
-import Control.Monad.Trans
 import Data.IORef
 import Data.Typeable
 import Data.Char (isAlphaNum)
 import Network.URI (escapeURIString)
 
 import IDE.Core.State
+import IDE.SymbolNavigation
 import IDE.Pane.SourceBuffer
+import IDE.TextEditor (EditorIter(..))
 import IDE.Utils.GUIUtils (openBrowser,controlIsPressed)
 import Graphics.UI.Gtk.SourceView
+import Control.Monad.IO.Class (MonadIO(..))
 
 
 -- | An info pane description
@@ -109,12 +112,23 @@ instance RecoverablePane IDEInfo InfoState IDEM where
 
             sw <- scrolledWindowNew Nothing Nothing
             containerAdd sw descriptionView
+
+
+            createHyperLinkSupport descriptionView sw (\_ _ iter -> do
+                    (GtkEditorIter beg,GtkEditorIter en) <- reflectIDE (getIdentifierUnderCursorFromIter (GtkEditorIter iter, GtkEditorIter iter)) ideR
+                    return (beg, en)) (\_ shift' slice -> do
+                                        when (slice /= []) $ do
+                                            -- liftIO$ print ("slice",slice)
+                                            reflectIDE (triggerEventIDE
+                                                (SelectInfo slice shift')) ideR
+                                            return ()
+                                        )
+
             scrolledWindowSetPolicy sw PolicyAutomatic PolicyAutomatic
 
             boxPackStart ibox sw PackGrow 10
 
 
-
             --openType
             currentDescr' <- newIORef idDescr
 #if MIN_VERSION_gtk(0,10,5)
@@ -131,7 +145,7 @@ instance RecoverablePane IDEInfo InfoState IDEM where
                     symbol  <- textBufferGetText buf l r True
                     when (controlIsPressed e)
                         (reflectIDE (do
-                            triggerEventIDE (SelectInfo symbol)
+                            triggerEventIDE (SelectInfo symbol False)
                             return ()) ideR)
                     return False)
             return (Just info,[ConnectC cid])
@@ -159,7 +173,7 @@ setInfo identifierDescr = do
     liftIO $ do
         writeIORef (currentDescr info) (Just identifierDescr)
         tb <- get (descriptionView info) textViewBuffer
-        textBufferSetText tb (show (Present identifierDescr))
+        textBufferSetText tb (show (Present identifierDescr) ++ "\n")   -- EOL for text iters to work
     recordInfoHistory (Just identifierDescr) oldDescr
 
 getInfoCont ::  IDEM (Maybe (Descr))
