@@ -1,5 +1,5 @@
-{-# OPTIONS_GHC -XScopedTypeVariables -XDeriveDataTypeable -XMultiParamTypeClasses
-    -XTypeSynonymInstances -XParallelListComp #-}
+{-# LANGUAGE FlexibleInstances, ScopedTypeVariables, DeriveDataTypeable,
+             MultiParamTypeClasses, TypeSynonymInstances, ParallelListComp #-}
 --
 -- Module      :  IDE.Pane.Log
 -- Copyright   :  (c) Juergen Nicklisch-Franken, Hamish Mackenzie
@@ -32,7 +32,6 @@ module IDE.Pane.Log (
 import Data.Typeable (Typeable(..))
 import IDE.Core.State
 import Graphics.UI.Gtk.Gdk.Events
-import Control.Monad.Trans (liftIO)
 import IDE.Pane.SourceBuffer (markRefInSourceBuf,selectSourceBuf)
 import System.IO
 import Prelude hiding (catch)
@@ -40,7 +39,7 @@ import Control.Exception hiding (try)
 import IDE.ImportTool
        (addPackage, parseHiddenModule, addImport, parseNotInScope,
         resolveErrors)
-import IDE.System.Process (runInteractiveProcess, ProcessHandle)
+import IDE.Utils.Tool (runInteractiveProcess, ProcessHandle)
 import Graphics.UI.Gtk
        (textBufferSetText, textViewScrollToMark,
         textBufferGetIterAtLineOffset, textViewScrollMarkOnscreen,
@@ -65,6 +64,7 @@ import Graphics.UI.Gtk
         ScrolledWindow, TextView, Menu, AttrOp(..), set,
         TextWindowType(..), ShadowType(..), PolicyType(..),
         priorityDefaultIdle, idleAdd)
+import Control.Monad.IO.Class (MonadIO(..))
 
 
 -------------------------------------------------------------------------------
