@@ -1,5 +1,5 @@
-{-# OPTIONS_GHC -XDeriveDataTypeable -XMultiParamTypeClasses
-    -XScopedTypeVariables -XTypeSynonymInstances #-}
+{-# LANGUAGE FlexibleInstances, DeriveDataTypeable, MultiParamTypeClasses,
+             ScopedTypeVariables, TypeSynonymInstances #-}
 -----------------------------------------------------------------------------
 --
 -- Module      :  IDE.Pane.Modules
@@ -29,7 +29,6 @@ module IDE.Pane.Modules (
 import Graphics.UI.Gtk hiding (get)
 import Graphics.UI.Gtk.Gdk.Events
 import Data.Maybe
-import Control.Monad.Reader
 import qualified Data.Map as Map
 import Data.Map (Map)
 import Data.Tree
@@ -60,6 +59,10 @@ import IDE.Metainfo.Provider
 import System.Log.Logger (infoM)
 import Default (Default(..))
 import IDE.Workspaces (packageTry_)
+import Control.Monad.IO.Class (MonadIO(..))
+import Control.Monad (when)
+import Control.Monad.Trans.Class (MonadTrans(..))
+import Control.Monad.Trans.Reader (ask)
 
 -- | A modules pane description
 --
@@ -295,8 +298,8 @@ instance RecoverablePane IDEModules ModulesState IDEM where
                 return ()
             return (Just modules,[ConnectC cid1,ConnectC cid2, ConnectC cid3])
 
-selectIdentifier :: Descr -> IDEAction
-selectIdentifier idDescr = do
+selectIdentifier :: Descr -> Bool -> IDEAction
+selectIdentifier idDescr openSource= do
     systemScope     <- getSystemInfo
     workspaceScope  <- getWorkspaceInfo
     packageScope    <- getPackageInfo
@@ -308,6 +311,7 @@ selectIdentifier idDescr = do
                         Just sc -> do
                             when (fst currentScope < sc) (setScope (sc,snd currentScope))
                             selectIdentifier' (modu pm) (dscName idDescr)
+    when openSource (goToDefinition idDescr)
 
 scopeForDescr :: PackModule -> Maybe (GenScope,GenScope) ->
     Maybe (GenScope,GenScope) -> Maybe GenScope -> Maybe Scope
