@@ -1,5 +1,5 @@
-{-# OPTIONS_GHC -XRecordWildCards -XTypeSynonymInstances -XMultiParamTypeClasses
-    -XDeriveDataTypeable #-}
+{-# LANGUAGE FlexibleInstances, RecordWildCards, TypeSynonymInstances,
+             MultiParamTypeClasses, DeriveDataTypeable #-}
 -----------------------------------------------------------------------------
 --
 -- Module      :  IDE.Pane.Trace
@@ -23,7 +23,6 @@ module IDE.Pane.Trace (
 import Graphics.UI.Gtk
 import Data.Typeable (Typeable(..))
 import IDE.Core.State
-import Control.Monad.Reader
 import IDE.Package (tryDebug_)
 import IDE.Debug
     (debugForward, debugBack, debugCommand')
@@ -48,6 +47,9 @@ import Graphics.UI.Gtk.Gdk.Events (Event(..))
 import Graphics.UI.Gtk.General.Enums (MouseButton(..))
 import System.Log.Logger (debugM)
 import IDE.Workspaces (packageTry_)
+import qualified Data.Enumerator.List as EL (consume)
+import Control.Monad.Trans.Class (MonadTrans(..))
+import Control.Monad.IO.Class (MonadIO(..))
 
 -- | A debugger pane description
 --
@@ -161,7 +163,9 @@ fillTraceList = packageTry_ $ do
     mbTraces     <- lift getPane
     case mbTraces of
         Nothing -> return ()
-        Just tracePane -> tryDebug_ $ debugCommand' ":history" (\to -> liftIO $ postGUIAsync $ do
+        Just tracePane -> tryDebug_ $ debugCommand' ":history" $ do
+            to <- EL.consume
+            liftIO $ postGUIAsync $ do
                 let parseRes = parse tracesParser "" (selectString to)
                 r <- case parseRes of
                         Left err     -> do
@@ -173,8 +177,8 @@ fillTraceList = packageTry_ $ do
                                                             then h{thSelected = True}
                                                             else h) r
                 mapM_ (insertTrace (tracepoints tracePane))
-                    (zip r' [0..length r']))
-    where
+                    (zip r' [0..length r'])
+  where
     insertTrace treeStore (tr,index)  = treeStoreInsert treeStore [] index tr
 
 selectString :: [ToolOutput] -> String
