@@ -40,6 +40,7 @@ module IDE.TextEditor (
 ,   endUserAction
 ,   getEndIter
 ,   getInsertMark
+,   getInsertIter
 ,   getIterAtLine
 ,   getIterAtMark
 ,   getIterAtOffset
@@ -129,20 +130,33 @@ module IDE.TextEditor (
 ,   onKeyPress
 ,   onKeyRelease
 ,   onLookupInfo
+,   onMotionNotify
+,   onLeaveNotify
 ,   onPopulatePopup
 ) where
 
 import Prelude hiding(getChar, getLine)
 import Data.Char (isAlphaNum, isSymbol)
-import Data.Maybe (fromJust)
+import Data.Maybe (fromJust, maybeToList)
+import Data.IORef
 import Control.Monad (when)
-import Control.Monad.Reader (liftIO, ask)
 import Control.Applicative ((<$>))
 
 import qualified Graphics.UI.Gtk as Gtk hiding(afterToggleOverwrite)
 import qualified Graphics.UI.Gtk.SourceView as Gtk
+import Graphics.UI.Gtk.Multiline.TextTagTable
+import qualified Graphics.UI.Gtk.Multiline.TextBuffer as Gtk
+import qualified  Graphics.UI.Gtk.Multiline.TextView as Gtk
+import qualified Graphics.UI.Gtk.Scrolling.ScrolledWindow
+import Graphics.UI.Gtk.Multiline.TextTag
+import Graphics.UI.Gtk.Gdk.Cursor
+--import Graphics.Rendering.Pango
+import System.Glib.Attributes
+import System.Glib.MainLoop
+import qualified Graphics.UI.Gtk.Multiline.TextView as Gtk
 import qualified Graphics.UI.Gtk.Gdk.Events as GtkOld
 import System.Glib.Attributes (AttrOp(..))
+import System.GIO.File.ContentType (contentTypeGuess)
 
 #ifdef LEKSAH_WITH_YI
 import qualified Yi as Yi hiding(withBuffer)
@@ -154,8 +168,11 @@ import Control.Monad (unless)
 
 import IDE.Core.State
 import IDE.Utils.GUIUtils(controlIsPressed)
-
-
+import Paths_leksah (getDataDir)
+import System.FilePath ((</>))
+import Control.Monad.IO.Class (liftIO)
+import Control.Monad.Trans.Reader (ask)
+import qualified Control.Monad.Reader as Gtk (liftIO)
 
 -- Data types
 data EditorBuffer = GtkEditorBuffer Gtk.SourceBuffer
@@ -188,6 +205,7 @@ data EditorTag = GtkEditorTag Gtk.TextTag
     | YiEditorTag
 #endif
 
+
 #ifdef LEKSAH_WITH_YI
 withYiBuffer' :: Yi.BufferRef -> Yi.BufferM a -> IDEM a
 withYiBuffer' b f = liftYi $ Yi.liftEditor $ Yi.withGivenBuffer0 b f
@@ -211,9 +229,17 @@ iterFromYiBuffer b f = iterFromYiBuffer' (Yi.fBufRef b) f
 -- Buffer
 newGtkBuffer :: Maybe FilePath -> String -> IDEM EditorBuffer
 newGtkBuffer mbFilename contents = liftIO $ do
-    lm     <- Gtk.sourceLanguageManagerNew
-    mbLang <- case mbFilename of
-        Just filename -> Gtk.sourceLanguageManagerGuessLanguage lm (Just filename) Nothing
+    lm      <- Gtk.sourceLanguageManagerNew
+    dataDir <- getDataDir
+    oldPath <- Gtk.sourceLanguageManagerGetSearchPath lm
+    Gtk.sourceLanguageManagerSetSearchPath lm (Just $ (dataDir </> "language-specs") : oldPath)
+    mbLang  <- case mbFilename of
+        Just filename -> do
+            guess <- contentTypeGuess filename contents (length contents)
+            Gtk.sourceLanguageManagerGuessLanguage lm (Just filename) $
+                case guess of
+                    (True, _)  -> Nothing
+                    (False, t) -> Just t
         Nothing       -> Gtk.sourceLanguageManagerGuessLanguage lm Nothing (Just "text/x-haskell")
     buffer <- case mbLang of
         Just sLang -> Gtk.sourceBufferNewWithLanguage sLang
@@ -395,6 +421,18 @@ getSelectionBounds (YiEditorBuffer b) = withYiBuffer b $ do
             mkYiIter b (Yi.regionEnd region))
 #endif
 
+getInsertIter :: EditorBuffer -> IDEM EditorIter
+getInsertIter (GtkEditorBuffer sb) = liftIO $ do
+    insertMark <- Gtk.textBufferGetInsert sb
+    insertIter <- Gtk.textBufferGetIterAtMark sb insertMark
+    return (GtkEditorIter insertIter)
+#ifdef LEKSAH_WITH_YI
+getInsertIter (YiEditorBuffer b) = withYiBuffer b $ do
+    insertMark <- Yi.insMark <$> Yi.askMarks
+    mkYiIter b <$> Yi.getMarkPointB insertMark
+#endif
+
+
 getSlice :: EditorBuffer
             -> EditorIter
             -> EditorIter
@@ -1074,22 +1112,34 @@ onCompletion (GtkEditorView sv) start cancel = do
     ideR <- ask
     (GtkEditorBuffer sb) <- getBuffer (GtkEditorView sv)
     liftIO $ do
+        -- when multiple afterBufferInsertText are called quickly,
+        -- we cancel previous idle action which was not processed,
+        -- its handler is stored here.
+        -- Paste operation is example of such sequential events (each word!).
+        lastHandler <- newIORef Nothing
         id1 <- sb `Gtk.afterBufferInsertText` \iter text -> do
-            let isIdent a = isAlphaNum a || a == '\'' || a == '_' || a == '.'
-            let isOp    a = isSymbol   a || a == ':'  || a == '\\' || a == '*' || a == '/' || a == '-'
-                                         || a == '!'  || a == '@' || a == '%' || a == '&' || a == '?'
-            if (all isIdent text) || (all isOp text)
-                then do
-                    hasSel <- Gtk.textBufferHasSelection sb
-                    if not hasSel
+            mapM_ idleRemove =<< maybeToList <$> readIORef lastHandler
+            writeIORef lastHandler =<< Just <$> do
+                (flip idleAdd) priorityDefault $ do
+                    let isIdent a = isAlphaNum a || a == '\'' || a == '_' || a == '.'
+                    let isOp    a = isSymbol   a || a == ':'  || a == '\\' || a == '*' || a == '/' || a == '-'
+                                                 || a == '!'  || a == '@' || a == '%' || a == '&' || a == '?'
+                    if (all isIdent text) || (all isOp text)
                         then do
-                            (iterC, _) <- Gtk.textBufferGetSelectionBounds sb
-                            atC <- Gtk.textIterEqual iter iterC
-                            when atC $ reflectIDE start ideR
-                        else
+                            hasSel <- Gtk.textBufferHasSelection sb
+                            if not hasSel
+                                then do
+                                    (iterC, _) <- Gtk.textBufferGetSelectionBounds sb
+                                    atC <- Gtk.textIterEqual iter iterC
+                                    when atC $ reflectIDE start ideR
+                                    return False
+                                else do
+                                    reflectIDE cancel ideR
+                                    return False
+                        else do
                             reflectIDE cancel ideR
-                else
-                    reflectIDE cancel ideR
+                            return False
+            return ()
 #if MIN_VERSION_gtk(0,10,5)
         id2 <- sv `Gtk.on` Gtk.moveCursor $ \_ _ _ -> reflectIDE cancel ideR
 #else
@@ -1112,7 +1162,7 @@ onKeyPress (GtkEditorView sv) f = do
             name        <- Gtk.eventKeyName
             modifier    <- Gtk.eventModifier
             keyVal      <- Gtk.eventKeyVal
-            liftIO $ reflectIDE (f name modifier keyVal) ideR
+            Gtk.liftIO $ reflectIDE (f name modifier keyVal) ideR
         return [ConnectC id1]
 #ifdef LEKSAH_WITH_YI
 onKeyPress (YiEditorView v) f = do
@@ -1126,6 +1176,47 @@ onKeyPress (YiEditorView v) f = do
         return [ConnectC id1]
 #endif
 
+onMotionNotify :: EditorView
+              -> (Double -> Double -> [Gtk.Modifier] -> IDEM Bool)
+              -> IDEM [Connection]
+onMotionNotify (GtkEditorView sv) f = do
+    ideR <- ask
+    liftIO $ do
+        id1 <- sv `Gtk.on` Gtk.motionNotifyEvent $ do
+            (ex,ey)     <- Gtk.eventCoordinates
+            modifier    <- Gtk.eventModifier
+            Gtk.liftIO $ reflectIDE (f ex ey modifier) ideR
+        return [ConnectC id1]
+#ifdef LEKSAH_WITH_YI
+onMotionNotify (YiEditorView v) f = do
+    ideR <- ask
+    liftIO $ do
+        id1 <- (Yi.drawArea v) `Gtk.on` Gtk.motionNotifyEvent $ do
+            (ex,ey)     <- Gtk.eventCoordinates
+            modifier    <- Gtk.eventModifier
+            liftIO $ reflectIDE (f ex ey modifier) ideR
+        return [ConnectC id1]
+#endif
+
+onLeaveNotify :: EditorView
+              -> (IDEM Bool)
+              -> IDEM [Connection]
+onLeaveNotify (GtkEditorView sv) f = do
+    ideR <- ask
+    liftIO $ do
+        id1 <- sv `Gtk.on` Gtk.leaveNotifyEvent $ do
+            Gtk.liftIO $ reflectIDE (f) ideR
+        return [ConnectC id1]
+#ifdef LEKSAH_WITH_YI
+onLeaveNotify (YiEditorView v) f = do
+    ideR <- ask
+    liftIO $ do
+        id1 <- (Yi.drawArea v) `Gtk.on` Gtk.leaveNotifyEvent $ do
+            liftIO $ reflectIDE (f) ideR
+        return [ConnectC id1]
+#endif
+
+
 onKeyRelease :: EditorView
                 -> (String -> [Gtk.Modifier] -> Gtk.KeyVal -> IDEM Bool)
                 -> IDEM [Connection]
@@ -1136,7 +1227,7 @@ onKeyRelease (GtkEditorView sv) f = do
             name        <- Gtk.eventKeyName
             modifier    <- Gtk.eventModifier
             keyVal      <- Gtk.eventKeyVal
-            liftIO $ reflectIDE (f name modifier keyVal) ideR
+            Gtk.liftIO $ reflectIDE (f name modifier keyVal) ideR
         return [ConnectC id1]
 #ifdef LEKSAH_WITH_YI
 onKeyRelease (YiEditorView v) f = do
@@ -1188,4 +1279,3 @@ onPopulatePopup (YiEditorView v) f = do
 #endif
 
 
-
