@@ -23,6 +23,8 @@ module IDE.Utils.GUIUtils (
 
 ,   getBackgroundBuildToggled
 ,   setBackgroundBuildToggled
+,   getRunUnitTests
+,   setRunUnitTests
 ,   getMakeModeToggled
 ,   setMakeModeToggled
 ,   getDebugToggled
@@ -33,13 +35,14 @@ module IDE.Utils.GUIUtils (
 ,   controlIsPressed
 
 ,   stockIdFromType
+,   mapControlCommand
+
 ) where
 
 import Graphics.UI.Gtk
-import IDE.System.Process
+import IDE.Utils.Tool (runProcess)
 import Data.Maybe (fromJust, isJust)
 import Control.Monad
-import Control.Monad.Trans(MonadIO,liftIO)
 import IDE.Core.State
 --import Graphics.UI.Gtk.Selectors.FileChooser
 --    (FileChooserAction(..))
@@ -51,6 +54,7 @@ import Graphics.UI.Gtk.Gdk.EventM (Modifier(..))
 #else
 import Graphics.UI.Gtk.Gdk.Enums (Modifier(..))
 #endif
+import Control.Monad.IO.Class (liftIO)
 
 chooseDir :: Window -> String -> Maybe FilePath -> IO (Maybe FilePath)
 chooseDir window prompt mbFolder = do
@@ -170,6 +174,16 @@ setBackgroundBuildToggled b = do
     ui <- getUIAction "ui/toolbar/BuildToolItems/BackgroundBuild" castToToggleAction
     liftIO $ toggleActionSetActive ui b
 
+getRunUnitTests :: PaneMonad alpha => alpha  (Bool)
+getRunUnitTests = do
+    ui <- getUIAction "ui/toolbar/BuildToolItems/RunUnitTests" castToToggleAction
+    liftIO $ toggleActionGetActive ui
+
+setRunUnitTests :: PaneMonad alpha => Bool -> alpha ()
+setRunUnitTests b = do
+    ui <- getUIAction "ui/toolbar/BuildToolItems/RunUnitTests" castToToggleAction
+    liftIO $ toggleActionSetActive ui b
+
 getMakeModeToggled :: PaneMonad alpha => alpha  (Bool)
 getMakeModeToggled = do
     ui <- getUIAction "ui/toolbar/BuildToolItems/MakeMode" castToToggleAction
@@ -212,5 +226,11 @@ stockIdFromType Field           =   "ide_slot"
 stockIdFromType Method          =   "ide_method"
 stockIdFromType _               =   "ide_other"
 
+-- maps control key for Macos
+#if defined(darwin_HOST_OS)
+mapControlCommand Alt = Control
+#endif
+mapControlCommand a = a
+
 
 
