@@ -20,15 +20,16 @@ module IDE.Utils.ServerConnection (
 import IDE.Core.State
 import Network (connectTo,PortID(..))
 import Network.Socket (PortNumber(..))
-import IDE.System.Process(runProcess)
+import IDE.Utils.Tool (runProcess)
 import GHC.Conc(threadDelay)
-import Control.Monad.Trans(liftIO)
 import System.IO
 import Control.Exception (SomeException(..), catch)
 import Prelude hiding(catch)
 import Control.Concurrent(forkIO)
 import Graphics.UI.Gtk(postGUIAsync)
 import Control.Event(triggerEvent)
+import Control.Monad.IO.Class (MonadIO(..))
+import System.Log.Logger (getLevel, getRootLogger)
 
 doServerCommand :: ServerCommand -> (ServerAnswer -> IDEM alpha) -> IDEAction
 doServerCommand command cont = do
@@ -69,7 +70,12 @@ doServerCommand command cont = do
 
 startServer :: Int -> IO ()
 startServer port = do
-    runProcess "leksah-server" ["--server=" ++ show port, "+RTS", "-N2", "-RTS"]
+    logger <- getRootLogger
+    let verbosity = case getLevel logger of
+                        Just level -> ["--verbosity=" ++ show level]
+                        Nothing    -> []
+    runProcess "leksah-server"
+        (["--server=" ++ show port, "+RTS", "-N2", "-RTS"] ++ verbosity)
         Nothing Nothing Nothing Nothing Nothing
     return ()
 
