@@ -27,8 +27,12 @@ module IDE.Workspaces (
 ,   workspaceAddPackage'
 ,   workspaceRemovePackage
 ,   workspacePackageNew
+,   workspaceTryQuiet
+,   workspaceNewHere
 ,   packageTry
 ,   packageTry_
+,   packageTryQuiet
+,   packageTryQuiet_
 
 ,   backgroundMake
 ,   makePackage
@@ -36,11 +40,9 @@ module IDE.Workspaces (
 ) where
 
 import IDE.Core.State
-import Control.Monad.Trans (liftIO)
 import Graphics.UI.Editor.Parameters
     (Parameter(..), (<<<-), paraName, emptyParams)
-import Control.Monad (unless, when, liftM)
-import Control.Monad.Reader (lift)
+import Control.Monad (forM_, unless, when, liftM)
 import Data.Maybe (isJust,fromJust )
 import IDE.Utils.GUIUtils
     (chooseFile, chooseSaveFile)
@@ -77,7 +79,6 @@ import Graphics.UI.Gtk.Windows.Dialog (ResponseId(..))
 import Graphics.UI.Gtk.General.Structs (ResponseId(..))
 #endif
 import Control.Exception (SomeException(..))
-import Control.Monad.Reader.Class (ask)
 import qualified Data.Map as  Map (empty)
 import IDE.Pane.SourceBuffer (fileOpenThis, fileCheckAll, belongsToPackage)
 import qualified System.IO.UTF8 as UTF8 (writeFile)
@@ -86,7 +87,11 @@ import Graphics.UI.Gtk.General.Enums (WindowPosition(..))
 import Control.Applicative ((<$>))
 import IDE.Build
 import IDE.Utils.FileUtils(myCanonicalizePath)
-
+import Control.Monad.Trans.Reader (ask)
+import Control.Monad.IO.Class (MonadIO(..))
+import Control.Monad.Trans.Class (lift)
+import qualified Data.Set as Set (toList)
+import Distribution.PackageDescription (hsSourceDirs)
 
 setWorkspace :: Maybe Workspace -> IDEAction
 setWorkspace mbWs = do
@@ -289,24 +294,20 @@ workspacePackageNew = do
 constructAndOpenMainModule :: Maybe IDEPackage -> IDEAction
 constructAndOpenMainModule Nothing = return ()
 constructAndOpenMainModule (Just idePackage) =
-    case (ipdMain idePackage, ipdSrcDirs idePackage) of
-        ([target],[path]) -> do
-            mbPD <- getPackageDescriptionAndPath
-            case mbPD of
-                Just (pd,_) -> do
-                    liftIO $ createDirectoryIfMissing True path
-                    alreadyExists <- liftIO $ doesFileExist (path </> target)
-                    if alreadyExists
-                        then do
-                            ideMessage Normal "Main file already exists"
-                            fileOpenThis (path </> target)
-                        else do
-                            template <- liftIO $ getModuleTemplate pd (dropExtension target)
-                                "    main" "main = putStrLn \"Hello World!\""
+    forM_ (ipdMain idePackage) $ \(target, bi, isTest) -> do
+        mbPD <- getPackageDescriptionAndPath
+        case mbPD of
+            Just (pd,_) -> do
+                case hsSourceDirs bi of
+                    path:_ -> do
+                        liftIO $ createDirectoryIfMissing True path
+                        alreadyExists <- liftIO $ doesFileExist (path </> target)
+                        unless alreadyExists $ do
+                            template <- liftIO $ getModuleTemplate "main" pd "Main" "" ""
                             liftIO $ UTF8.writeFile (path </> target) template
                             fileOpenThis (path </> target)
-                Nothing     -> ideMessage Normal "No package description"
-        _ -> return ()
+                    _ -> return ()
+            Nothing     -> ideMessage Normal "No package description"
 
 workspaceAddPackage :: WorkspaceAction
 workspaceAddPackage = do
@@ -517,7 +518,7 @@ workspaceClean = do
     settings <- lift $ do
         prefs' <- readIDE prefs
         return (defaultMakeSettings prefs')
-    makePackages settings (wsPackages ws) MoClean MoClean
+    makePackages settings (wsPackages ws) MoClean MoClean moNoOp
 
 workspaceMake :: WorkspaceAction
 workspaceMake = do
@@ -527,8 +528,10 @@ workspaceMake = do
         return ((defaultMakeSettings prefs'){
                     msMakeMode           = True,
                     msBackgroundBuild    = False})
-    makePackages settings (wsPackages ws) (MoComposed [MoConfigure,MoBuild,MoInstall])
-        (MoComposed [MoConfigure,MoBuild,MoInstall])
+    let steps = if msRunUnitTests settings
+                    then [MoConfigure,MoBuild,MoTest,MoCopy,MoRegister]
+                    else [MoConfigure,MoBuild,MoCopy,MoRegister]
+    makePackages settings (wsPackages ws) (MoComposed steps) (MoComposed steps) MoMetaInfo
 
 backgroundMake :: IDEAction
 backgroundMake = catchIDE (do
@@ -546,10 +549,14 @@ backgroundMake = catchIDE (do
                 let settings = defaultMakeSettings prefs
                 if msSingleBuildWithoutLinking settings &&  not (msMakeMode settings)
                     then workspaceTryQuiet_ $
-                        makePackages settings modifiedPacks MoBuild (MoComposed [])
-                    else workspaceTryQuiet_ $
-                        makePackages settings modifiedPacks (MoComposed [MoBuild,MoInstall])
-                                        (MoComposed [MoConfigure,MoBuild,MoInstall])
+                        makePackages settings modifiedPacks MoBuild (MoComposed []) moNoOp
+                    else do
+                        let steps = if msRunUnitTests settings
+                                        then [MoBuild,MoTest,MoCopy,MoRegister]
+                                        else [MoBuild,MoCopy,MoRegister]
+                        workspaceTryQuiet_ $
+                            makePackages settings modifiedPacks (MoComposed steps)
+                                        (MoComposed (MoConfigure:steps)) MoMetaInfo
     )
     (\(e :: SomeException) -> sysMessage Normal (show e))
 
@@ -566,9 +573,14 @@ makePackage = do
         Just ws -> lift $
             if msSingleBuildWithoutLinking settings &&  not (msMakeMode settings)
                 then runWorkspace
-                        (makePackages settings [p] MoBuild (MoComposed [])) ws
-                else runWorkspace
+                        (makePackages settings [p] MoBuild (MoComposed []) moNoOp) ws
+                else do
+                    let steps = if msRunUnitTests settings
+                                    then [MoBuild,MoTest,MoCopy,MoRegister]
+                                    else [MoBuild,MoCopy,MoRegister]
+                    runWorkspace
                         (makePackages settings [p]
-                        (MoComposed [MoBuild,MoInstall])
-                        (MoComposed [MoConfigure,MoBuild,MoInstall])) ws
+                        (MoComposed steps)
+                        (MoComposed (MoConfigure:steps))
+                        MoMetaInfo) ws
 
